@@ -31,8 +31,6 @@
 
 #define BUFF_SIZE	 16384
 
-#define NPAD_BYTES(o) ((4 - (o % 4)) % 4)
-
 typedef enum {
 	INPUT_FROM_FD,
 	INPUT_FROM_MEMORY
@@ -61,7 +59,7 @@ int get_cpiohdr(unsigned char *buf, struct filehdr *fhdr)
 	return 0;
 }
 
-static int fill_buffer(int fd, unsigned char *buf, unsigned int nbytes, unsigned long *offs,
+static int _fill_buffer(int fd, unsigned char *buf, unsigned int nbytes, unsigned long *offs,
 	uint32_t *checksum, void *dgst)
 {
 	ssize_t len;
@@ -94,6 +92,13 @@ static int fill_buffer(int fd, unsigned char *buf, unsigned int nbytes, unsigned
 	return count;
 }
 
+
+int fill_buffer(int fd, unsigned char *buf, unsigned int nbytes)
+{
+	unsigned long offs = 0;
+	return _fill_buffer(fd, buf, nbytes, &offs, NULL, NULL);
+}
+
 /*
  * Read padding that could exists between the cpio trailer and the end-of-file.
  * cpio aligns the file to 512 bytes
@@ -237,7 +242,7 @@ static int input_step(void *state, void *buffer, size_t size)
 	}
 	switch (s->source) {
 	case INPUT_FROM_FD:
-		ret = fill_buffer(s->fdin, buffer, size, s->offs, &s->checksum, s->dgst);
+		ret = _fill_buffer(s->fdin, buffer, size, s->offs, &s->checksum, s->dgst);
 		if (ret < 0) {
 			return ret;
 		}
@@ -514,8 +519,8 @@ static int __swupdate_copy(int fdin, unsigned char *inbuf, void *out, size_t nby
 
 	if (encrypted) {
 		aes_key = get_aes_key();
-		if (imgivt) {
-			if (!strlen(imgivt) || !is_hex_str(imgivt) || ascii_to_bin(ivtbuf, sizeof(ivtbuf), imgivt)) {
+		if (imgivt && strlen(imgivt)) {
+			if (!is_hex_str(imgivt) || ascii_to_bin(ivtbuf, sizeof(ivtbuf), imgivt)) {
 				ERROR("Invalid image ivt");
 				return -EINVAL;
 			}
@@ -671,7 +676,7 @@ static int __swupdate_copy(int fdin, unsigned char *inbuf, void *out, size_t nby
 	}
 
 	if (!inbuf) {
-		ret = fill_buffer(fdin, buffer, NPAD_BYTES(*offs), offs, checksum, NULL);
+		ret = _fill_buffer(fdin, buffer, NPAD_BYTES(*offs), offs, checksum, NULL);
 		if (ret < 0)
 			DEBUG("Padding bytes are not read, ignoring");
 	}
@@ -759,7 +764,7 @@ int copyimage(void *out, struct img_type *img, writeimage callback)
 int extract_cpio_header(int fd, struct filehdr *fhdr, unsigned long *offset)
 {
 	unsigned char buf[sizeof(fhdr->filename)];
-	if (fill_buffer(fd, buf, sizeof(struct new_ascii_header), offset, NULL, NULL) < 0)
+	if (_fill_buffer(fd, buf, sizeof(struct new_ascii_header), offset, NULL, NULL) < 0)
 		return -EINVAL;
 	if (get_cpiohdr(buf, fhdr) < 0) {
 		ERROR("CPIO Header corrupted, cannot be parsed");
@@ -773,13 +778,13 @@ int extract_cpio_header(int fd, struct filehdr *fhdr, unsigned long *offset)
 		return -EINVAL;
 	}
 
-	if (fill_buffer(fd, buf, fhdr->namesize , offset, NULL, NULL) < 0)
+	if (_fill_buffer(fd, buf, fhdr->namesize , offset, NULL, NULL) < 0)
 		return -EINVAL;
 	buf[fhdr->namesize] = '\0';
 	strlcpy(fhdr->filename, (char *)buf, sizeof(fhdr->filename));
 
 	/* Skip filename padding, if any */
-	if (fill_buffer(fd, buf, (4 - (*offset % 4)) % 4, offset, NULL, NULL) < 0)
+	if (_fill_buffer(fd, buf, (4 - (*offset % 4)) % 4, offset, NULL, NULL) < 0)
 		return -EINVAL;
 
 	return 0;
