@@ -33,6 +33,7 @@
 #include "progress.h"
 #include "pctl.h"
 #include "swupdate_vars.h"
+#include "lua_util.h"
 
 /*
  * function returns:
@@ -186,6 +187,24 @@ static int prepare_var_script(struct dict *dict, const char *script)
 	return 0;
 }
 
+static int generate_swversions(struct swupdate_cfg *cfg)
+{
+	FILE *fp;
+	struct sw_version *swver;
+	struct swver *sw_ver_list = &cfg->installed_sw_list;
+
+	fp = fopen(cfg->output_swversions, "w");
+	if (!fp)
+		return -EACCES;
+
+	LIST_FOREACH(swver, sw_ver_list, next) {
+		fprintf(fp, "%s\t\t%s\n", swver->name, swver->version);
+	}
+	fclose(fp);
+
+	return 0;
+}
+
 static int update_bootloader_env(struct swupdate_cfg *cfg, const char *script)
 {
 	int ret = 0;
@@ -215,7 +234,7 @@ static int update_swupdate_vars(struct swupdate_cfg *cfg, const char *script)
 	return ret;
 }
 
-static int run_prepost_scripts(struct imglist *list, script_fn type)
+int run_prepost_scripts(struct imglist *list, script_fn type)
 {
 	int ret;
 	struct img_type *img;
@@ -278,6 +297,44 @@ int install_single_image(struct img_type *img, bool dry_run)
 	return ret;
 }
 
+static int update_installed_image_version(struct swver *sw_ver_list,
+		struct img_type *img)
+{
+	struct sw_version *swver;
+	struct sw_version *swcomp;
+
+	if (!sw_ver_list)
+		return false;
+
+	LIST_FOREACH(swver, sw_ver_list, next) {
+		/*
+		 * If component is already installed, update the version
+		 */
+		if (!strncmp(img->id.name, swver->name, sizeof(img->id.name))) {
+			strncpy(swver->version, img->id.version, sizeof(img->id.version));
+			return true;
+		}
+	}
+
+	if (!strlen(img->id.version))
+		return false;
+
+	/*
+	 * No previous version of this component is installed. Create a new entry.
+	 */
+	swcomp = (struct sw_version *)calloc(1, sizeof(struct sw_version));
+	if (!swcomp) {
+		ERROR("Could not create new version entry.");
+		return false;
+	}
+
+	strlcpy(swcomp->name, img->id.name, sizeof(swcomp->name));
+	strlcpy(swcomp->version, img->id.version, sizeof(swcomp->version));
+	LIST_INSERT_HEAD(sw_ver_list, swcomp, next);
+
+	return true;
+}
+
 /*
  * streamfd: file descriptor if it is required to extract
  *           images from the stream (update from file)
@@ -369,6 +426,8 @@ int install_images(struct swupdate_cfg *sw)
 
 		close(img->fdin);
 
+		update_installed_image_version(&sw->installed_sw_list, img);
+
 		if (dropimg)
 			free_image(img);
 
@@ -408,6 +467,16 @@ int install_images(struct swupdate_cfg *sw)
 
 	ret |= run_prepost_scripts(&sw->bootscripts, POSTINSTALL);
 
+	/*
+	 * Should we generate a list with installed software?
+	 */
+	if (strlen(sw->output_swversions)) {
+		ret |= generate_swversions(sw);
+		if (ret) {
+			ERROR("%s cannot be opened", sw->output_swversions);
+		}
+	}
+
 	return ret;
 }
 
@@ -478,6 +547,14 @@ void cleanup_files(struct swupdate_cfg *software) {
 	dict_drop_db(&software->bootloader);
 	dict_drop_db(&software->vars);
 
+	/*
+	 * Drop Lua State if instantiated
+	 */
+	if (software->lua_state) {
+		unregister_session_handlers();
+		lua_exit(software->lua_state);
+		software->lua_state = NULL;
+	}
 	if (asprintf(&fn, "%s%s", TMPDIR, BOOT_SCRIPT_SUFFIX) != ENOMEM_ASPRINTF) {
 		remove_sw_file(fn);
 		free(fn);
