@@ -204,33 +204,39 @@ static void lua_report_exception(lua_State *L)
 	} while (*s++);
 }
 
-int run_lua_script(const char *script, const char *function, char *parms)
+int run_lua_script(lua_State *L, const char *script, bool load, const char *function, char *parms)
 {
 	int ret;
 	const char *output;
 
-	lua_State *L = luaL_newstate(); /* opens Lua */
-	luaL_openlibs(L); /* opens the standard libraries */
-	luaL_requiref(L, "swupdate", luaopen_swupdate, 1 );
-
-	if (luaL_loadfile(L, script)) {
-		ERROR("ERROR loading %s", script);
-		lua_close(L);
-		return -1;
+	if (!L) {
+		ERROR("Lua script must be executed, but no valid Lua state was set");
+		return -EINVAL;
 	}
 
-	ret = lua_pcall(L, 0, 0, 0);
-	if (ret) {
-		LUAstackDump(L);
-		ERROR("ERROR preparing Lua script %s %d",
-			script, ret);
-		lua_close(L);
-		return -1;
+	if (load) {
+		TRACE("Loading Lua %s script", script);
+		if (luaL_loadfile(L, script)) {
+			ERROR("ERROR loading %s", script);
+			return -1;
+		}
+
+		ret = lua_pcall(L, 0, 0, 0);
+		if (ret) {
+			LUAstackDump(L);
+			ERROR("ERROR preparing Lua script %s %d",
+				script, ret);
+			return -1;
+		}
+	}
+
+	if (!function) {
+		WARN("Script was loaded, no function was set to be executed !");
+		return 0;
 	}
 
 	lua_getglobal(L, function);
 	if(!lua_isfunction(L,lua_gettop(L))) {
-		lua_close(L);
 		TRACE("Script : no %s in %s script, exiting", function, script);
 		return 0;
 	}
@@ -241,7 +247,6 @@ int run_lua_script(const char *script, const char *function, char *parms)
 	if (lua_pcall(L, 1, 2, 0)) {
 		LUAstackDump(L);
 		ERROR("ERROR Calling Lua script %s", script);
-		lua_close(L);
 		return -1;
 	}
 
@@ -257,8 +262,6 @@ int run_lua_script(const char *script, const char *function, char *parms)
 		TRACE("Script output: %s script end", output);
 	}
 
-	lua_close(L);
-
 	return ret;
 }
 
@@ -906,6 +909,9 @@ static int l_set_bootenv(lua_State *L) {
 	const char *name = luaL_checkstring(L, 1);
 	const char *value = luaL_checkstring(L, 2);
 
+	if (!bootenv)
+		return -1;
+
 	if (name && strlen(name))
 		dict_set_value(bootenv, name, value ? value : "");
 	lua_pop(L, 2);
@@ -967,6 +973,28 @@ l_get_hw_exit:
 	return 1;
 }
 
+static int l_get_emmc_bootpart(lua_State *L)
+{
+	const char *device = luaL_checkstring(L, 1);
+	int active = -1, fd;
+
+	if (!device) {
+		lua_pushnil(L);
+		return 1;
+	}
+
+	fd = open(device, O_RDONLY);
+	if (fd < 0) {
+		lua_pushnil(L);
+		return 1;
+	}
+
+	active = emmc_get_active_bootpart(fd);
+	close(fd);
+	lua_pushinteger(L, active);
+	return 1;
+}
+
 static void lua_push_enum(lua_State *L, const char *name, int value)
 {
 	lua_pushstring(L, name);
@@ -1188,6 +1216,7 @@ static const luaL_Reg l_swupdate[] = {
         { "get_hw", l_get_hw },
         { "getversion", lua_get_swupdate_version },
         { "progress", lua_notify_progress },
+        { "emmcbootpart", l_get_emmc_bootpart },
         { NULL, NULL }
 };
 
@@ -1269,8 +1298,6 @@ static int luaopen_swupdate(lua_State *L)
 	return 1;
 }
 
-static lua_State *gL = NULL;
-
 /**
  * @brief wrapper to call the Lua function
  *
@@ -1291,56 +1318,66 @@ static int l_handler_wrapper(struct img_type *img, void *data,
 	int res = 0;
 	lua_Number result;
 	int l_func_ref;
+	lua_State *L;
+	struct installer_handler *hnd;
+	hnd = find_handler(img);
 
-	if (!gL || !img || !data) {
+	if (!img || !data || !hnd) {
+		return -1;
+	}
+
+	if (hnd->noglobal) {
+		L = img->L;
+	} else {
+		ERROR("Calling session Lua handler in non-session context.");
 		return -1;
 	}
 
 	if (img->bootloader) {
-		lua_getglobal(gL, "swupdate");
-		if (!lua_istable(gL, -1)) {
+		lua_getglobal(L, "swupdate");
+		if (!lua_istable(L, -1)) {
 			ERROR("Lua stack corrupted.");
 			return -1;
 		}
-		struct dict **udbootenv = lua_newuserdata(gL, sizeof(struct dict*));
+		struct dict **udbootenv = lua_newuserdata(L, sizeof(struct dict*));
 		*udbootenv = img->bootloader;
-		luaL_setfuncs(gL, l_swupdate_bootenv, 1);
-		lua_pop(gL, 1);
+		luaL_setfuncs(L, l_swupdate_bootenv, 1);
+		lua_pop(L, 1);
 	}
 
 	l_func_ref = *((int*)data);
 	/* get the callback function */
-	lua_rawgeti(gL, LUA_REGISTRYINDEX, l_func_ref );
-	image2table(gL, img);
+	lua_rawgeti(L, LUA_REGISTRYINDEX, l_func_ref );
+	image2table(L, img);
 
 	switch (scriptfn) {
 	case PREINSTALL:
-		lua_pushstring(gL, "preinst");
+		lua_pushstring(L, "preinst");
 		break;
 	case POSTINSTALL:
-		lua_pushstring(gL, "postinst");
+		lua_pushstring(L, "postinst");
 		break;
 	default:
-		lua_pushnil(gL);
+		lua_pushnil(L);
 		break;
 	}
 
-	if (LUA_OK != (res = lua_pcall(gL, 2, 1, 0))) {
+	if (LUA_OK != (res = lua_pcall(L, 2, 1, 0))) {
 		ERROR("Error %d while executing the Lua callback: %s",
-			  res, lua_tostring(gL, -1));
-		lua_pop(gL, 1);
+			  res, lua_tostring(L, -1));
+		lua_pop(L, 1);
 		return -1;
 	}
 
 	 /* retrieve result */
-	if (!lua_isnumber(gL, -1)) {
+	if (!lua_isnumber(L, -1)) {
 		ERROR("Lua Handler must return a number");
-		lua_pop(gL, 1);
+		lua_pop(L, 1);
 		return -1;
 	}
 
-	result = lua_tonumber(gL, -1);
-	lua_pop(gL, 1);
+	result = lua_tonumber(L, -1);
+	lua_pop(L, 1);
 	TRACE("[Lua handler] returned: %d",(int)result);
 
 	return (int) result;
@@ -1369,6 +1406,7 @@ static int l_other_handler_wrapper(struct img_type *img, void *data)
  */
 static int l_register_handler( lua_State *L ) {
 	int *l_func_ref = malloc(sizeof(int));
+
 	if(!l_func_ref) {
 		ERROR("Lua handler: unable to allocate memory");
 		lua_pop(L, 2);
@@ -1394,16 +1432,32 @@ static int l_register_handler( lua_State *L ) {
 		}
 
 		const char *handler_desc = luaL_checkstring(L, 1);
+
 		/* store the callback function in registry */
 		*l_func_ref = luaL_ref (L, LUA_REGISTRYINDEX);
 		/* cleanup stack */
 		lua_pop (L, 1);
 
-		register_handler(handler_desc,
-				 (mask & SCRIPT_HANDLER) ?
-				 l_script_handler_wrapper :
-				 l_other_handler_wrapper,
-				 mask, l_func_ref);
+		register_session_handler(handler_desc,
+				(mask & SCRIPT_HANDLER) ?
+				l_script_handler_wrapper :
+				l_other_handler_wrapper,
+				mask, l_func_ref);
+
+		/* add newly registered handler to current Lua stack */
+		lua_getglobal(L, "swupdate");
+		lua_pushliteral(L, "handler");
+		lua_gettable(L, -2);
+		if (lua_istable(L, -1)) {
+			lua_pushstring(L, handler_desc);
+			lua_pushnumber(L, 1);
+			lua_settable(L, -3);
+		} else {
+			TRACE("swupdate.handler Table absent in current Lua "
+			      "context, not inserting handler '%s'", handler_desc);
+		}
+		lua_pop(L, 2);
+
 		return 0;
 	}
 }
@@ -1457,7 +1511,7 @@ call_handler_exit:
 	return 2;
 }
 
-int lua_handlers_init(void)
+int lua_handlers_init(lua_State *L)
 {
 	static const char location[] =
 #if defined(CONFIG_EMBEDDED_LUA_HANDLER)
@@ -1467,26 +1521,33 @@ int lua_handlers_init(void)
 #endif
 	int ret = -1;
 
-	gL = luaL_newstate();
-	if (gL) {
-		/* prime gL as LUA_TYPE_HANDLER */
-		lua_pushlightuserdata(gL, (void*)LUA_TYPE_HANDLER);
-		lua_setglobal(gL, "SWUPDATE_LUA_TYPE");
+	if (L) {
+		/* prime L as LUA_TYPE_HANDLER */
+		lua_pushlightuserdata(L, (void*)LUA_TYPE_HANDLER);
+		lua_setglobal(L, "SWUPDATE_LUA_TYPE");
 		/* load standard libraries */
-		luaL_openlibs(gL);
-		luaL_requiref( gL, "swupdate", luaopen_swupdate, 1 );
-		lua_pop(gL, 1); /* remove unused copy left on stack */
+		luaL_openlibs(L);
+		/* load / fore-reload swupdate module */
+		lua_getglobal(L, "package");
+		lua_pushliteral(L, "loaded");
+		lua_gettable(L, -2);
+		lua_pushstring(L, "swupdate");
+		lua_pushnil(L);
+		lua_settable(L, -3);
+		lua_pop(L, 2);
+		luaL_requiref(L, "swupdate", luaopen_swupdate, 1 );
+		lua_pop(L, 1); /* remove unused copy left on stack */
 		/* try to load Lua handlers for the swupdate system */
 #if defined(CONFIG_EMBEDDED_LUA_HANDLER)
-		ret = (luaL_loadbuffer(gL, EMBEDDED_LUA_SRC_START, EMBEDDED_LUA_SRC_END-EMBEDDED_LUA_SRC_START, "LuaHandler") ||
-		       lua_pcall(gL, 0, LUA_MULTRET, 0));
+		ret = (luaL_loadbuffer(L, EMBEDDED_LUA_SRC_START, EMBEDDED_LUA_SRC_END-EMBEDDED_LUA_SRC_START, "LuaHandler") ||
+		       lua_pcall(L, 0, LUA_MULTRET, 0));
 #else
-		ret = luaL_dostring(gL, "require (\"swupdate_handlers\")");
+		ret = luaL_dostring(L, "require (\"swupdate_handlers\")");
 #endif
 		if (ret != 0) {
 			INFO("%s Lua handler(s) not found.", location);
-			lua_report_exception(gL);
-			lua_pop(gL, 1);
+			lua_report_exception(L);
+			lua_pop(L, 1);
 		} else {
 			INFO("%s Lua handler(s) found and loaded.", location);
 		}
@@ -1497,7 +1558,7 @@ int lua_handlers_init(void)
 	return ret;
 }
 
-lua_State *lua_parser_init(const char *buf, struct dict *bootenv)
+lua_State *lua_session_init(struct dict *bootenv)
 {
 	lua_State *L = luaL_newstate(); /* opens Lua */
 
@@ -1513,16 +1574,32 @@ lua_State *lua_parser_init(const char *buf, struct dict *bootenv)
 	luaL_setfuncs(L, l_swupdate_bootenv, 1);
 	lua_pop(L, 1); /* remove unused copy left on stack */
 
-	if (luaL_loadstring(L, buf) || lua_pcall(L, 0, 0, 0)) {
-		LUAstackDump(L);
-		ERROR("ERROR preparing Lua embedded script in parser");
-		lua_close(L);
-		return NULL;
-	}
+	lua_handlers_init(L);
 
 	return L;
 }
 
+int lua_init(void)
+{
+	lua_State *L = luaL_newstate();
+	int res = lua_handlers_init(L);
+	print_registered_handlers(false);
+	unregister_session_handlers();
+	lua_close(L);
+	return res;
+}
+
+int lua_load_buffer(lua_State *L, const char *buf)
+{
+	if (luaL_loadstring(L, buf) || lua_pcall(L, 0, 0, 0)) {
+		LUAstackDump(L);
+		ERROR("ERROR loading Lua code");
+		return 1;
+	}
+
+	return 0;
+}
+
 int lua_parser_fn(lua_State *L, const char *fcn, struct img_type *img)
 {
 	int ret = -1;
@@ -1564,3 +1641,39 @@ int lua_parser_fn(lua_State *L, const char *fcn, struct img_type *img)
 
 	return ret;
 }
+
+int lua_handler_fn(lua_State *L, const char *fcn, const char *parms)
+{
+	int ret = -1;
+
+	lua_getglobal(L, fcn);
+	if(!lua_isfunction(L, lua_gettop(L))) {
+		lua_pop(L, 1);
+		TRACE("Script : no %s in script, exiting", fcn);
+		return -1;
+	}
+
+	/*
+	 * passing arguments
+	 */
+	lua_pushstring(L, parms);
+
+	ret = lua_pcall(L, 1, 1, 0);
+	if (ret || !lua_isinteger(L, -1)) {
+		LUAstackDump(L);
+		ERROR("ERROR Calling Lua %s", fcn);
+		return -1;
+	}
+
+	if (loglevel >= DEBUGLEVEL)
+		LUAstackDump(L);
+
+	ret = lua_tointeger(L, -1);
+
+	lua_pop(L, 1); /* clear stack */
+
+	if (loglevel >= DEBUGLEVEL)
+		TRACE("Script returns %d", ret);
+
+	return ret;
+}
