@@ -18,9 +18,27 @@
 #include "bsdqueue.h"
 #include "util.h"
 #include "parselib.h"
+#include "parselib-private.h"
 
 #define MAX_URL_LENGTH 2048
 
+static json_type map_field_type(field_type_t type)
+{
+	switch (type) {
+	case TYPE_INT:
+	case TYPE_INT64:
+		return json_type_int;
+	case TYPE_STRING:
+		return json_type_string;
+	case TYPE_BOOL:
+		return json_type_boolean;
+	case TYPE_FLOAT:
+		return json_type_double;
+	default: /* not supported in SWUpdate */
+		return json_type_null;
+	}
+}
+
 json_object *find_json_recursive_node(json_object *root, const char **names)
 {
 	json_object *node = root;
@@ -102,10 +120,12 @@ const char *get_field_string_json(json_object *e, const char *path)
 	return NULL;
 }
 
-void get_value_json(json_object *e, void *dest)
+static void get_value_json(json_object *e, void *dest, field_type_t expected_type)
 {
 	enum json_type type;
 	type = json_object_get_type(e);
+	if (type != map_field_type(expected_type))
+		return;
 	switch (type) {
 	case json_type_boolean:
 		*(unsigned int *)dest = json_object_get_boolean(e);
@@ -141,15 +161,15 @@ bool is_field_numeric_json(json_object *e, const char *path)
 	       type == json_type_double;
 }
 
-void get_field_json(json_object *e, const char *path, void *dest)
+void get_field_json(json_object *e, const char *path, void *dest, field_type_t type)
 {
 	json_object *fld = NULL;
 
 	if (path) {
 		if (json_object_object_get_ex(e, path, &fld))
-			get_value_json(fld, dest);
+			get_value_json(fld, dest, type);
 	} else {
-		get_value_json(e, dest);
+		get_value_json(e, dest, type);
 	}
 }
 
