@@ -4,6 +4,7 @@
  *
  * SPDX-License-Identifier:     GPL-2.0-only
  */
+#include <errno.h>
 #include <unistd.h>
 #include <stdbool.h>
 #include <stdlib.h>
@@ -20,12 +21,15 @@ int channel_settings(void *elem, void *data)
 	char tmp[128];
 	channel_data_t *chan = (channel_data_t *)data;
 
-	get_field(LIBCFG_PARSER, elem, "retry",
+	GET_FIELD_INT(LIBCFG_PARSER, elem, "retry",
 		&chan->retries);
 
 	GET_FIELD_STRING_RESET(LIBCFG_PARSER, elem, "max-download-speed", tmp);
-	if (strlen(tmp))
+	if (strlen(tmp)) {
 		chan->max_download_speed = (unsigned int)ustrtoull(tmp, NULL, 10);
+		if (errno)
+			WARN("max-download-speed setting %s: ustrtoull failed", tmp);
+	}
 
 	GET_FIELD_STRING_RESET(LIBCFG_PARSER, elem, "retrywait", tmp);
 	if (strlen(tmp))
@@ -37,6 +41,9 @@ int channel_settings(void *elem, void *data)
 	GET_FIELD_STRING_RESET(LIBCFG_PARSER, elem, "sslkey", tmp);
 	if (strlen(tmp))
 		SETSTRING(chan->sslkey, tmp);
+	GET_FIELD_STRING_RESET(LIBCFG_PARSER, elem, "sslkeypassword", tmp);
+	if (strlen(tmp))
+		SETSTRING(chan->sslkeypassword, tmp);
 	GET_FIELD_STRING_RESET(LIBCFG_PARSER, elem, "ciphers", tmp);
 	if (strlen(tmp))
 		SETSTRING(chan->ciphers, tmp);
