@@ -24,7 +24,6 @@
 #include <sys/statvfs.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <ctype.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
@@ -39,12 +38,12 @@
 #include <sys/mman.h>
 #include "delta_handler.h"
 #include "multipart_parser.h"
-#include "installer.h"
 #include "zchunk_range.h"
 #include "chained_handler.h"
 #include "swupdate_image.h"
 
 #define DEFAULT_MAX_RANGES	150	/* Apache has default = 200 */
+#define BUFF_SIZE		16384
 
 const char *handlername = "delta";
 void delta_handler(void);
@@ -327,10 +326,13 @@ static int delta_retrieve_attributes(struct img_type *img, struct hnd_priv *priv
 	char *srcsize;
 	srcsize = dict_get_value(&img->properties, "source-size");
 	if (srcsize) {
-		if (!strcmp(srcsize, "detect"))
+		if (!strcmp(srcsize, "detect")) {
 			priv->detectsrcsize = true;
-		else
+		} else {
 			priv->srcsize = ustrtoull(srcsize, NULL, 10);
+			if (errno)
+				WARN("source-size %s: ustrotull failed", srcsize);
+		}
 	}
 
 	char *zckloglevel = dict_get_value(&img->properties, "zckloglevel");
@@ -470,34 +472,58 @@ static void zck_log_toswupdate(const char *function, zck_log_type lt,
 
 /*
  * Create a zck Index from a file
+ *
+ * If maxbytes has been set, it acts as a limit for the input data.
+ * If not (i.e. maxbytes==0), all the file/dev available data is used.
  */
 static bool create_zckindex(zckCtx *zck, int fd, size_t maxbytes)
 {
-	const size_t bufsize = 16384;
-	char *buf = malloc(bufsize);
-	ssize_t n;
-	int ret;
+	size_t bufsize = BUFF_SIZE;
+	char *buf = malloc(BUFF_SIZE);
+	ssize_t n = 0;
+	size_t count = 0;
+	bool rstatus = true;
 
 	if (!buf) {
 		ERROR("OOM creating temporary buffer");
 		return false;
 	}
 	while ((n = read(fd, buf, bufsize)) > 0) {
-		ret = zck_write(zck, buf, n);
-		if (ret < 0) {
+		if (zck_write(zck, buf, n) < 0) {
 			ERROR("ZCK returns %s", zck_get_error(zck));
 			free(buf);
 			return false;
 		}
-		if (maxbytes && n > maxbytes)
-			break;
+
+		if(maxbytes) {
+			/* Keep count only if maxbytes has been set and it's significant*/
+			count += n;
+
+			/* Stop if limit is reached*/
+			if (count >= maxbytes)
+				break;
+
+			/* Be sure read up to maxbytes limit next time */
+			if (BUFF_SIZE > (maxbytes - count))
+				bufsize = maxbytes - count;
+		}
 	}
 
 	free(buf);
+	if (zck_end_chunk(zck) < 0) {
+		ERROR("ZCK failed to create chunk boundary: %s", zck_get_error(zck));
+		return false;
+	}
 
-	return true;
+	if(n < 0) {
+		ERROR("Error occurred while reading data : %s", strerror(errno));
+		rstatus = false;
+	}
+
+	return rstatus;
 }
 
+
 /*
  * Chunks must be retrieved from network, prepare an send
  * a request for the downloader
@@ -734,7 +760,7 @@ static bool copy_network_chunks(zckChunk **dstChunk, struct hnd_priv *priv)
 					return false;
 				}
 			}
-			if ((answer->type == RANGE_DATA)) {
+			if (answer->type == RANGE_DATA) {
 				priv->dwlstate = WAITING_FOR_BOUNDARY;
 			}
 			break;
@@ -763,7 +789,7 @@ static bool copy_network_chunks(zckChunk **dstChunk, struct hnd_priv *priv)
 			if (!read_and_validate_package(priv))
 				return false;
 			answer = priv->answer;
-			if ((answer->type == RANGE_COMPLETED)) {
+			if (answer->type == RANGE_COMPLETED) {
 				priv->dwlstate = END_TRANSFER;
 			} else if (!fill_buffers_list(priv))
 				return false;
