@@ -293,9 +293,13 @@ static int install_remote_swu(struct img_type *img,
 	struct dict_list_elem *url;
 	struct dict_list *urls;
 	int index = 0;
+	const char *fn_parse_answer = NULL;
 	pthread_attr_t attr;
 	int thread_ret = -1;
 
+	/* Reset list of connections */
+	LIST_INIT(&priv.conns);
+
 	/*
 	 * A single SWU can contains encrypted artifacts,
 	 * but the SWU itself cannot be encrypted.
@@ -317,8 +321,16 @@ static int install_remote_swu(struct img_type *img,
 		return -EINVAL;
 	}
 
-	/* Reset list of connections */
-	LIST_INIT(&priv.conns);
+	/*
+	 * Check if a custom function is set to parse the
+	 * answer form the Webserver
+	 */
+	fn_parse_answer = dict_get_value(&img->properties, "parser-function");
+	if (fn_parse_answer && !img->L) {
+		ERROR("Custom parser requires to enable Lua, exiting..");
+		ret = FAILURE;
+		goto handler_exit;
+	}
 
 	/* initialize CURL */
 	ret = curl_global_init(CURL_GLOBAL_DEFAULT);
@@ -344,9 +356,14 @@ static int install_remote_swu(struct img_type *img,
 			goto handler_exit;
 		}
 
+		/*
+		 * Set parameters to each connection structure
+		 */
 		conn->url = url->value;
 		conn->total_bytes = img->size;
 		conn->SWUpdateStatus = IDLE;
+		conn->L = img->L;
+		conn->fnparser = fn_parse_answer;
 
 		LIST_INSERT_HEAD(&priv.conns, conn, next);
 
