@@ -15,9 +15,9 @@
 #define SW_DESCRIPTION_FILENAME	CONFIG_SWDESCRIPTION
 #endif
 
-typedef int (*parser_fn)(struct swupdate_cfg *swcfg, const char *filename);
+typedef int (*parser_fn)(struct swupdate_cfg *swcfg, const char *filename, char **error);
 
 int parse(struct swupdate_cfg *swcfg, const char *filename);
-int parse_cfg (struct swupdate_cfg *swcfg, const char *filename);
-int parse_json(struct swupdate_cfg *swcfg, const char *filename);
-int parse_external(struct swupdate_cfg *swcfg, const char *filename);
+int parse_cfg(struct swupdate_cfg *swcfg, const char *filename, char **error);
+int parse_json(struct swupdate_cfg *swcfg, const char *filename, char **error);
+int parse_external(struct swupdate_cfg *swcfg, const char *filename, char **error);
