@@ -36,12 +36,6 @@ struct extproc {
 
 LIST_HEAD(proclist, extproc);
 
-enum {
-	SCRIPT_NONE,
-	SCRIPT_PREINSTALL,
-	SCRIPT_POSTINSTALL
-};
-
 struct swupdate_parms {
 	bool dry_run;
 	char software_set[SWUPDATE_GENERAL_STRING_SIZE];
@@ -55,6 +49,7 @@ struct swupdate_cfg {
 	bool bootloader_transaction_marker;
 	bool bootloader_state_marker;
 	char output[SWUPDATE_GENERAL_STRING_SIZE];
+	char output_swversions[SWUPDATE_GENERAL_STRING_SIZE];
 	char publickeyfname[SWUPDATE_GENERAL_STRING_SIZE];
 	char aeskeyfname[SWUPDATE_GENERAL_STRING_SIZE];
 	char postupdatecmd[SWUPDATE_GENERAL_STRING_SIZE];
@@ -65,6 +60,7 @@ struct swupdate_cfg {
 	char mtdblacklist[SWUPDATE_GENERAL_STRING_SIZE];
 	char forced_signer_name[SWUPDATE_GENERAL_STRING_SIZE];
 	char namespace_for_vars[SWUPDATE_GENERAL_STRING_SIZE];
+	void *lua_state;
 	bool syslog_enabled;
 	bool no_downgrading;
 	bool no_reinstalling;
@@ -72,7 +68,7 @@ struct swupdate_cfg {
 	bool no_state_marker;
 	bool reboot_required;
 	bool check_max_version;
-	int verbose;
+	bool verbose;
 	int loglevel;
 	int cert_purpose;
 	struct hw_type hw;
