@@ -12,6 +12,7 @@
 #include "bsdqueue.h"
 #include "globals.h"
 #include "swupdate_dict.h"
+#include "lua_util.h"
 
 typedef enum {
 	FLASH,
@@ -53,14 +54,21 @@ struct img_type {
 	skip_t skip;
 	int provided;
 	int compressed;
-	int preserve_attributes; /* whether to preserve attributes in archives */
+	bool preserve_attributes; /* whether to preserve attributes in archives */
 	bool is_encrypted;
 	char ivt_ascii[33];
 	int install_directly;
 	int is_script;
 	int is_partitioner;
 	struct dict properties;
-	struct dict *bootloader; /* pointer to swupdate_cfg's bootloader dict for handlers to modify */
+
+	/*
+	 * Pointers to global structures
+	 * that are alive during an installation. They can be used by handlers
+	 */
+	struct dict *bootloader;/* pointer to swupdate_cfg's bootloader dict for handlers to modify */
+	lua_State *L;		/* pointer to swupdate_cfg's LUa state created by parser */
+
 	long long partsize;
 	int fdin;	/* Used for streaming file */
 	off_t offset;	/* offset in cpio file */
