@@ -33,7 +33,6 @@
 #define AES_256_KEY_LEN	32
 
 #define HWID_REGEXP_PREFIX	"#RE:"
-#define SWUPDATE_ALIGN(A,S)    (((A) + (S) - 1) & ~((S) - 1))
 
 #define BOOTVAR_TRANSACTION "recovery_status"
 
@@ -272,3 +271,7 @@ int swupdate_umount(const char *dir);
 
 /* Date / Time utilities */
 char *swupdate_time_iso8601(struct timeval *tv);
+
+/* eMMC functions */
+int emmc_write_bootpart(int fd, int bootpart);
+int emmc_get_active_bootpart(int fd);
