@@ -195,7 +195,7 @@ static bool get_common_fields(parsertype p, void *cfg, struct swupdate_cfg *swcf
 	} else {
 		swcfg->bootloader_state_marker = true;
 		if((setting = find_node(p, cfg, "bootloader_state_marker", swcfg)) != NULL) {
-			get_field(p, setting, NULL, &swcfg->bootloader_state_marker);
+			GET_FIELD_BOOL(p, setting, NULL, &swcfg->bootloader_state_marker);
 			TRACE("Setting bootloader state marker: %s",
 			      swcfg->bootloader_state_marker == true ? "true" : "false");
 		}
@@ -206,7 +206,7 @@ static bool get_common_fields(parsertype p, void *cfg, struct swupdate_cfg *swcf
 	} else {
 		swcfg->bootloader_transaction_marker = true;
 		if((setting = find_node(p, cfg, "bootloader_transaction_marker", swcfg)) != NULL) {
-			get_field(p, setting, NULL, &swcfg->bootloader_transaction_marker);
+			GET_FIELD_BOOL(p, setting, NULL, &swcfg->bootloader_transaction_marker);
 			TRACE("Setting bootloader transaction marker: %s",
 			      swcfg->bootloader_transaction_marker == true ? "true" : "false");
 		}
@@ -217,7 +217,7 @@ static bool get_common_fields(parsertype p, void *cfg, struct swupdate_cfg *swcf
 	 */
 	swcfg->reboot_required = true;
 	if((setting = find_node(p, cfg, "reboot", swcfg)) != NULL) {
-		get_field(p, setting, NULL, &swcfg->reboot_required);
+		GET_FIELD_BOOL(p, setting, NULL, &swcfg->reboot_required);
 	}
 
 	TRACE("reboot_required %d", swcfg->reboot_required);
@@ -230,7 +230,6 @@ static bool get_common_fields(parsertype p, void *cfg, struct swupdate_cfg *swcf
 			TRACE("Output file set but not enabled with -o, ignored");
 		} else {
 			GET_FIELD_STRING(p, setting, NULL, swcfg->output);
-			get_field(p, setting, NULL, &swcfg->output);
 			TRACE("Incoming SWU stored : %s", swcfg->output);
 		}
 	}
@@ -381,6 +380,12 @@ static int is_image_higher(struct swver *sw_ver_list,
     return false;
 }
 
+static void set_img_globals(struct img_type *img, struct swupdate_cfg *sw)
+{
+	img->bootloader = &sw->bootloader;
+	img->L = sw->lua_state;
+}
+
 static int run_embscript(parsertype p, void *elem, struct img_type *img,
 			 lua_State *L, const char *embscript)
 {
@@ -422,7 +427,7 @@ static int parse_common_attributes(parsertype p, void *elem, struct img_type *im
 	 * multiplier suffixes are allowed
 	 */
 	if (is_field_numeric(p, elem, "offset")) {
-		get_field(p, elem, "offset", &offset);
+		GET_FIELD_INT64(p, elem, "offset", &offset);
 		image->seek = offset;
 	} else {
 		GET_FIELD_STRING(p, elem, "offset", seek_str);
@@ -444,13 +449,15 @@ static int parse_common_attributes(parsertype p, void *elem, struct img_type *im
 			return -1;
 		}
 	} else {
-		get_field(p, elem, "compressed", &image->compressed);
+		bool img_compressed = false;
+		GET_FIELD_BOOL(p, elem, "compressed", &img_compressed);
+		image->compressed = img_compressed ? COMPRESSED_TRUE : COMPRESSED_FALSE;
 	}
-	get_field(p, elem, "installed-directly", &image->install_directly);
-	get_field(p, elem, "preserve-attributes", &image->preserve_attributes);
-	get_field(p, elem, "install-if-different", &image->id.install_if_different);
-	get_field(p, elem, "install-if-higher", &image->id.install_if_higher);
-	get_field(p, elem, "encrypted", &image->is_encrypted);
+	GET_FIELD_BOOL(p, elem, "installed-directly", &image->install_directly);
+	GET_FIELD_BOOL(p, elem, "preserve-attributes", &image->preserve_attributes);
+	GET_FIELD_BOOL(p, elem, "install-if-different", &image->id.install_if_different);
+	GET_FIELD_BOOL(p, elem, "install-if-higher", &image->id.install_if_higher);
+	GET_FIELD_BOOL(p, elem, "encrypted", &image->is_encrypted);
 	GET_FIELD_STRING(p, elem, "ivt", image->ivt_ascii);
 
 	if (is_image_installed(&cfg->installed_sw_list, image)) {
@@ -515,7 +522,7 @@ static int _parse_partitions(parsertype p, void *cfg, void *setting, const char
 			return -1;
 		}
 
-		get_field(p, elem, "size", &partition->partsize);
+		GET_FIELD_INT64(p, elem, "size", &partition->partsize);
 
 		add_properties(p, elem, partition);
 
@@ -598,6 +605,8 @@ static int _parse_scripts(parsertype p, void *cfg, void *setting, const char **n
 
 		add_properties(p, elem, script);
 
+		set_img_globals(script, swcfg);
+
 		skip = run_embscript(p, elem, script, L, swcfg->embscript);
 		if (skip < 0) {
 			free_image(script);
@@ -863,7 +872,7 @@ static int _parse_images(parsertype p, void *cfg, void *setting, const char **no
 
 		add_properties(p, elem, image);
 
-		image->bootloader = &swcfg->bootloader;
+		set_img_globals(image, swcfg);
 
 		skip = run_embscript(p, elem, image, L, swcfg->embscript);
 		if (skip < 0) {
@@ -959,7 +968,7 @@ static int _parse_files(parsertype p, void *cfg, void *setting, const char **nod
 
 		add_properties(p, elem, file);
 
-		file->bootloader = &swcfg->bootloader;
+		set_img_globals(file, swcfg);
 
 		skip = run_embscript(p, elem, file, L, swcfg->embscript);
 		if (skip < 0) {
@@ -1015,15 +1024,24 @@ static int parser(parsertype p, void *cfg, struct swupdate_cfg *swcfg)
 		swcfg->embscript = get_field_string(p, scriptnode, NULL);
 	}
 
+	L = lua_session_init(&swcfg->bootloader);
+
 	if (swcfg->embscript) {
+		if (!L) {
+			ERROR("Required embedded script but no Lua not available");
+			return -1;
+		}
 		if (loglevel >= DEBUGLEVEL)
 			TRACE("Found Lua Software:\n%s", swcfg->embscript);
-		L = lua_parser_init(swcfg->embscript, &swcfg->bootloader);
-		if (!L) {
+		if (lua_load_buffer(L, swcfg->embscript)) {
 			ERROR("Required embedded script that cannot be loaded");
+			lua_close(L);
 			return -1;
 		}
 	}
+
+	swcfg->lua_state = L;
+
 	if (get_hw_revision(&swcfg->hw) < 0) {
 		TRACE("Hardware compatibility not found");
 	}
@@ -1042,9 +1060,6 @@ static int parser(parsertype p, void *cfg, struct swupdate_cfg *swcfg)
 	 */
 	parse_partitions(p, cfg, swcfg, L);
 
-	if (L)
-		lua_parser_exit(L);
-
 	if (LIST_EMPTY(&swcfg->images) &&
 	    LIST_EMPTY(&swcfg->scripts) &&
 	    LIST_EMPTY(&swcfg->bootloader)) {
@@ -1056,7 +1071,7 @@ static int parser(parsertype p, void *cfg, struct swupdate_cfg *swcfg)
 }
 
 #ifdef CONFIG_LIBCONFIG
-int parse_cfg (struct swupdate_cfg *swcfg, const char *filename)
+int parse_cfg(struct swupdate_cfg *swcfg, const char *filename, char **error)
 {
 	config_t cfg;
 	parsertype p = LIBCFG_PARSER;
@@ -1068,8 +1083,11 @@ int parse_cfg (struct swupdate_cfg *swcfg, const char *filename)
 	/* Read the file. If there is an error, report it and exit. */
 	DEBUG("Parsing config file %s", filename);
 	if(config_read_file(&cfg, filename) != CONFIG_TRUE) {
-		ERROR("%s:%d - %s\n", config_error_file(&cfg),
-			config_error_line(&cfg), config_error_text(&cfg));
+		if (asprintf(error, "%s:%d - %s\n", config_error_file(&cfg),
+			     config_error_line(&cfg), config_error_text(&cfg)) == ENOMEM_ASPRINTF) {
+			ERROR("OOM when caching error");
+			return -ENOMEM;
+		}
 		config_destroy(&cfg);
 		return -1;
 	}
@@ -1084,8 +1102,9 @@ int parse_cfg (struct swupdate_cfg *swcfg, const char *filename)
 	return ret;
 }
 #else
-int parse_cfg (struct swupdate_cfg __attribute__ ((__unused__)) *swcfg,
-		const char __attribute__ ((__unused__)) *filename)
+int parse_cfg(struct swupdate_cfg __attribute__((__unused__)) *swcfg,
+	      const char __attribute__((__unused__)) *filename,
+	      char __attribute__((__unused__)) **error)
 {
 	return -1;
 }
@@ -1093,7 +1112,7 @@ int parse_cfg (struct swupdate_cfg __attribute__ ((__unused__)) *swcfg,
 
 #define JSON_OBJECT_FREED 1
 
-int parse_json(struct swupdate_cfg *swcfg, const char *filename)
+int parse_json(struct swupdate_cfg *swcfg, const char *filename, char **error)
 {
 	int fd, ret;
 	struct stat stbuf;
@@ -1134,7 +1153,10 @@ int parse_json(struct swupdate_cfg *swcfg, const char *filename)
 
 	cfg = json_tokener_parse(string);
 	if (!cfg) {
-		ERROR("JSON File corrupted");
+		if (asprintf(error, "JSON File corrupted") == ENOMEM_ASPRINTF) {
+			ERROR("OOM when caching error");
+			return -ENOMEM;
+		}
 		free(string);
 		return -1;
 	}
