@@ -57,6 +57,7 @@ static struct option long_options[] = {
     {"server", required_argument, NULL, 'S'},
 	{"connection-timeout", required_argument, NULL, 's'},
 	{"custom-http-header", required_argument, NULL, 'a'},
+	{"identify", required_argument, NULL, '3'},
 	{"max-download-speed", required_argument, NULL, 'l'},
     {NULL, 0, NULL, 0}};
 
@@ -866,9 +867,13 @@ static void get_action_id_from_env(int *action_id)
 	 * Get the acction_id that corresponds to the done update if it was
 	 * stored.
 	 */
+	*action_id = -1;
 	char *action_str = swupdate_vars_get("action_id", NULL);
 	if (action_str) {
 		int tmp = ustrtoull(action_str, NULL, 10);
+		if (errno)
+			WARN("action_id %s: ustrtoull failed",
+			     action_str);
 		/*
 		 * action_id = 0 is invalid, then check it
 		 */
@@ -996,7 +1001,7 @@ static void *process_notification_thread(void *data)
 	for (;;) {
 		ipc_message msg;
 		bool data_avail = false;
-		int ret = ipc_get_status_timeout(&msg, 100);
+		int ret = ipc_get_status(&msg);
 
 		data_avail = ret > 0 && (strlen(msg.data.status.desc) != 0);
 
@@ -1059,6 +1064,10 @@ static void *process_notification_thread(void *data)
 
 		if (stop && !data_avail)
 			break;
+
+		// wait a bit for next message...
+		if (!data_avail)
+			usleep(100000);
 	}
 
 	pthread_mutex_unlock(&notifylock);
@@ -1692,6 +1701,7 @@ static void server_print_help(void)
 	    "\t  -s, --connection-timeout Set the server connection timeout (default: 300s).\n"
 	    "\t  -a, --custom-http-header <name> <value> Set custom HTTP header, "
 	    "appended to every HTTP request being sent.\n"
+	    "\t  --identify <name> <value> Set custom device attributes for Suricatta.\n"
 	    "\t  -n, --max-download-speed <limit>  Set download speed limit.\n"
 	    "\t                                    Example: -n 100k; -n 1M; -n 100; -n 1G\n",
 	    CHANNEL_DEFAULT_POLLING_INTERVAL, CHANNEL_DEFAULT_RESUME_TRIES,
@@ -1719,18 +1729,18 @@ static int server_hawkbit_settings(void *elem, void  __attribute__ ((__unused__)
 		mandatory_argument_count |= URL_BIT;
 	}
 
-	get_field(LIBCFG_PARSER, elem, "polldelay",
+	GET_FIELD_INT(LIBCFG_PARSER, elem, "polldelay",
 		&server_hawkbit.polling_interval);
 
-	get_field(LIBCFG_PARSER, elem, "initial-report-resend-period",
+	GET_FIELD_INT(LIBCFG_PARSER, elem, "initial-report-resend-period",
 		&server_hawkbit.initial_report_resend_period);
 
 	channel_settings(elem, &channel_data_defaults);
 
-	get_field(LIBCFG_PARSER, elem, "usetokentodwl",
+	GET_FIELD_BOOL(LIBCFG_PARSER, elem, "usetokentodwl",
 		&server_hawkbit.usetokentodwl);
 
-	get_field(LIBCFG_PARSER, elem, "connection-timeout",
+	GET_FIELD_INT(LIBCFG_PARSER, elem, "connection-timeout",
 		&channel_data_defaults.connection_timeout);
 
 	GET_FIELD_STRING_RESET(LIBCFG_PARSER, elem, "targettoken", tmp);
@@ -1891,9 +1901,23 @@ static server_op_res_t server_start(const char *fname, int argc, char *argv[])
 						argv[optind++]) < 0)
 				return SERVER_EINIT;
 			break;
+		case '3':
+			if (optind >= argc)
+				return SERVER_EINIT;
+
+			if (dict_insert_value(&server_hawkbit.configdata,
+						optarg,
+						argv[optind++]) < 0)
+				return SERVER_EINIT;
+			break;
 		case 'n':
 			channel_data_defaults.max_download_speed =
 				(unsigned int)ustrtoull(optarg, NULL, 10);
+			if (errno) {
+				ERROR("max-download-speed %s: ustrtoull failed",
+				      optarg);
+				return SERVER_EINIT;
+			}
 			break;
 		/* Ignore not recognized options, they can be already parsed by the caller */
 		case '?':
@@ -2008,11 +2032,12 @@ static server_op_res_t server_activation_ipc(ipc_message *msg)
 	int action_id = -1;
 	if (json_data) {
 		action_id = json_object_get_int(json_data);
-	} else {
+	}
+	if (action_id <= 0) { /* 0 is not a valid action id */
 		get_action_id_from_env(&action_id);
 	}
 
-	if (action_id < 0) {
+	if (action_id <= 0) {
 		ERROR("No action_id passed into JSON message and no action:_id in env");
 		return SERVER_EERR;
 	}
@@ -2039,6 +2064,10 @@ static server_op_res_t server_activation_ipc(ipc_message *msg)
 		return  SERVER_EERR;
 	}
 
+	if (!json_data) {
+		ERROR("No details are passed, they are mandatory.");
+		return SERVER_EERR;
+	}
 	int numdetails = json_object_array_length(json_data);
 	const char **details = (const char **)malloc((numdetails + 1) * (sizeof (char *)));
 	if(!details)
