@@ -233,6 +233,7 @@ int main(int argc, char **argv)
 	char *script = NULL;
 	bool wait_update = true;
 	bool disable_reboot = false;
+	bool redirected = false;
 
 	/* Process options with getopt */
 	while ((c = getopt_long(argc, argv, "cwprhs:e:q",
@@ -280,6 +281,8 @@ int main(int argc, char **argv)
 		snprintf(psplash_pipe_path, sizeof(psplash_pipe_path), "%s/psplash_fifo", rundir);
 	}
 	connfd = -1;
+	redirected = !isatty(fileno(stdout));
+
 	while (1) {
 		if (connfd < 0) {
 			connfd = progress_ipc_connect(opt_w);
@@ -336,7 +339,7 @@ int main(int argc, char **argv)
 		 * Be sure that string in message are Null terminated
 		 */
 		if (msg.infolen > 0) {
-			char *reboot_mode;
+			char reboot_mode[20] = { 0 };
 			int n, cause;
 
 			if (msg.infolen >= sizeof(msg.info) - 1) {
@@ -351,15 +354,14 @@ int main(int argc, char **argv)
 			 * will be added, JSON lib should be linked.
 			 * NOTE: Until then, the exact string format is imperative!
 			 */
-			n = sscanf(msg.info, "{\"%d\": { \"reboot-mode\" : \"%m[-a-z]\"}}",
-				   &cause, &reboot_mode);
+			n = sscanf(msg.info, "{\"%*d\": {\"%d\": { \"reboot-mode\" : \"%19[-a-z]\"}}}",
+				   &cause, reboot_mode);
 			if (n == 2) {
 				if (cause == CAUSE_REBOOT_MODE) {
 					if (!strcmp(reboot_mode, "no-reboot")) {
 						disable_reboot = true;
 					}
 				}
-				free(reboot_mode);
 			}
 		}
 		msg.cur_image[sizeof(msg.cur_image) - 1] = '\0';
@@ -385,6 +387,8 @@ int main(int argc, char **argv)
 						bar,
 						msg.cur_step, msg.nsteps, msg.cur_percent,
 						msg.cur_image, msg.dwl_percent, msg.dwl_bytes);
+					if (redirected)
+						fprintf(stdout, "\n");
 					fflush(stdout);
 				}
 
